"""
Client's main module.
"""


from pydantic import (
    HttpUrl,
    SecretStr,
    constr,
    validator,
)
from pydantic.dataclasses import dataclass


from .endpoints import Endpoint
from .endpoints.person import Person
from .endpoints.company import Company
from .endpoints.location import Location
from .endpoints.school import School
from .logger import get_logger
from .models import AutocompleteModel
from .requests import Request
from .settings import settings
from .utils import check_empty_parameters


logger = get_logger()


@dataclass
class PDLPY:
    """
    Client's main class. All methods derive from the instantiation of this
    class.

    Args:
        api_key (:obj:`str`, optional): The authentication
            API key for API calls.
        base_path (:obj:`str`, optional): PeopleDataLabs' API base URL.
        version (:obj:`str`, optional): PeopleDataLabs' API version.
            Will be used only if base_path has no value.
        log_level (:obj:`str`, optional): The logger level.
    """

    api_key: SecretStr = settings.api_key
    base_path: HttpUrl = None
    version: constr(regex=settings.version_re) = settings.version
    log_level: str = None

    @validator("api_key", pre=True, always=True)
    def api_key_not_none(cls, v):
        """
        Checks an API key is passed to the Client object.
        """
        if v is None:
            raise ValueError(
                "Please enter a value for API key, or define it either in"
                " your environment or .env file as PDL_API_KEY"
            )
        return v

    def __post_init__(self):
        """
        Sets the actual base_path and sets log_level globally across modules.
        """
        if self.base_path is None:
            self.base_path = settings.base_path + self.version
        if self.log_level is not None:
            settings.log_level = self.log_level
            logger.setLevel(self.log_level)

    @check_empty_parameters
    def autocomplete(self, **kwargs):
        """
        Calls PeopleDataLabs' autocomplete API.

        Args:
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        url = Endpoint(self.api_key, self.base_path).get_url(
            endpoint="autocomplete"
        )
        return Request(
            api_key=self.api_key,
            url=url,
            headers={
                "Accept-Encoding": "gzip",
                "User-Agent": "PDL-PYTHON-SDK",
                "Content-Type": "application/json",
            },
            params=kwargs,
            validator=AutocompleteModel,
        ).get()

    @property
    def company(self):
        """
        Calls API from the company section.
        """
        return Company(self.api_key, self.base_path)

    @property
    def location(self):
        """
        Calls API from the location section.
        """
        return Location(self.api_key, self.base_path)

    @property
    def school(self):
        """
        Calls API from the school section.
        """
        return School(self.api_key, self.base_path)

    @property
    def person(self):
        """
        Calls API from the person section.
        """
        return Person(self.api_key, self.base_path)



"""
Utility scripts which do not strictly belong to any other module.
"""


import functools

from pydantic import SecretStr

from .errors import EmptyParametersException


def check_empty_parameters(func):
    """
    Decorator for API request methods which checks if parameters are empty.
    """

    @functools.wraps(func)
    def _check(ref, *args, **kwargs):
        if not kwargs:
            raise EmptyParametersException
        return func(ref, *args, **kwargs)

    return _check


def json_defaults(value):
    """
    JSON default callback to serialize different types of data.
    """
    mapping = {SecretStr: str}
    return mapping[type(value)](value)



"""
Package to resolve endpoints from the People Data Labs' API.
"""


from typing import Type

from pydantic import (
    BaseModel,
    HttpUrl,
    SecretStr,
    StrictStr,
)
from pydantic.dataclasses import dataclass

from ..errors import InvalidEndpointError
from ..requests import Request
from ..utils import check_empty_parameters

headers = {
    "Accept-Encoding": "gzip",
    "User-Agent": "PDL-PYTHON-SDK",
    "Content-Type": "application/json",
}


@dataclass
class Endpoint:
    """
    Base class for all endpoints.

    Args:
        api_key (str): The authentication API key for API calls.
        base_path (str): PeopleDataLabs' API base URL.
        section: (:obj:`str`, optional): The section to prepend to the
            API endpoint.
    """

    api_key: SecretStr
    base_path: HttpUrl
    section: str = None

    def get_url(self, endpoint: str):
        """
        Forms the URL for the API call.

        Args:
            endpoint (str): The endpoint of the API to call.
        """
        url = self.base_path
        if self.section:
            url += "/" + self.section
        url += "/" + endpoint

        return url

    def __getattr__(self, method_name):
        """
        Raises InvalidEndpointError when an undefined method is called.
        """
        # pylint: disable=unused-argument
        def method(*args, **kwargs):
            cls_name = self.__class__.__name__
            raise InvalidEndpointError(
                f"Invalid method {method_name} called for section {cls_name}."
            )

        return method

    @check_empty_parameters
    def _bulk(self, model: Type[BaseModel], **kwargs):
        """
        Calls PeopleDataLabs' bulk enrichment API.

        Args:
            model: The model used for parameters validation.
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        url = self.get_url(endpoint="bulk")
        return Request(
            api_key=self.api_key,
            url=url,
            headers=headers,
            params=kwargs,
            validator=model,
        ).post()

    @check_empty_parameters
    def _cleaner(self, model: Type[BaseModel], **kwargs):
        """
        Calls PeopleDataLabs' cleaner API.

        Args:
            model: The model used for parameters validation.
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        url = self.get_url(endpoint="clean")
        return Request(
            api_key=self.api_key,
            url=url,
            headers=headers,
            params=kwargs,
            validator=model,
        ).get()

    @check_empty_parameters
    def _enrichment(self, model: Type[BaseModel], **kwargs):
        """
        Calls PeopleDataLabs' enrichment API.

        Args:
            model: The model used for parameters validation.
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        url = self.get_url(endpoint="enrich")
        return Request(
            api_key=self.api_key,
            url=url,
            headers=headers,
            params=kwargs,
            validator=model,
        ).get()

    @check_empty_parameters
    def _identify(self, model: Type[BaseModel], **kwargs):
        """
        Calls PeopleDataLabs' identify API.

        Args:
            model: The model used for parameters validation.
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        url = self.get_url(endpoint="identify")
        return Request(
            api_key=self.api_key,
            url=url,
            headers=headers,
            params=kwargs,
            validator=model,
        ).get()

    def _retrieve(
        self,
        model: Type[BaseModel],
        person_id: StrictStr,
        **kwargs,
    ):
        """
        Calls PeopleDataLabs' retrieve API.

        Args:
            model: The model used for parameters validation.
            person_id (str): The person's ID from the
                People Data Labs dataset;
            **kwargs: Additional parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        url = self.get_url(endpoint="retrieve")
        url += "/" + person_id
        return Request(
            api_key=self.api_key,
            url=url,
            headers=headers,
            params=kwargs,
            validator=model,
        ).get()

    @check_empty_parameters
    def _search(self, model: Type[BaseModel], **kwargs):
        """
        Calls PeopleDataLabs' search API.

        Args:
            model: The model used for parameters validation.
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        url = self.get_url(endpoint="search")
        return Request(
            api_key=self.api_key,
            url=url,
            headers=headers,
            params=kwargs,
            validator=model,
        ).post()



"""
Exceptions raised by the client.
"""


class EmptyParametersException(Exception):
    """
    Thrown when an API method is invoked with no parameters.
    """


class InvalidEndpointError(Exception):
    """
    Thrown when an endpoint is called for a section which does not support that
    endpoint.
    """



"""
Models for input parameters of the Company APIs.
"""


from typing import List, Optional, Union

from pydantic import (
    BaseModel,
    root_validator,
)

from . import (
    AdditionalParametersModel,
    BaseRequestModel,
    BaseSearchModel,
)


class CompanyBaseModel(BaseModel):
    """
    Base parameters model for the enrichment API.
    """

    country: Optional[str]
    locality: Optional[str]
    location: Optional[Union[List[str], str]]
    name: Optional[str]
    postal_code: Optional[str]
    profile: Optional[str]
    region: Optional[str]
    street_address: Optional[str]
    ticker: Optional[str]
    website: Optional[str]

    @root_validator(pre=True)
    def non_ambiguous(cls, v):
        """
        Checks that at leat one between 'name', 'ticker', 'website' and
        'profile' is given.
        """
        if not any(
            [
                v.get("name"),
                v.get("profile"),
                v.get("ticker"),
                v.get("website"),
            ]
        ):
            raise ValueError(
                "Company Enrichment API requires a non-ambiguous match."
                " See documentation @"
                " https://docs.peopledatalabs.com/docs/enrichment-api"
            )

        return v


class EnrichmentModel(
    BaseRequestModel, CompanyBaseModel, AdditionalParametersModel
):
    """
    Model for the enrichment API.
    """


class SearchModel(BaseSearchModel):
    """
    Search parameters model validator for Company search API.
    """


# pylint: disable=duplicate-code
class CleanerModel(BaseRequestModel):
    """
    Validation model for Company 'cleaner' API.
    """

    name: Optional[str]
    website: Optional[str]
    profile: Optional[str]

    @root_validator(pre=True)
    def at_least_one(cls, value):
        """
        Checks that at least one parameter is valued.
        """
        if not any(value.values()):
            raise ValueError(
                "At least one between 'name' 'website' or 'profile' is"
                " required. See documentation @"
                " https://docs.peopledatalabs.com/docs/cleaner-apis#parameters"
            )

        return value



"""
Requests module.

All requests are handled here.
"""


import json
from typing import Dict, Type

from pydantic import (
    BaseModel,
    HttpUrl,
    SecretStr,
)
from pydantic.dataclasses import dataclass
import requests

from . import utils
from .logger import get_logger


logger = get_logger("requests")


@dataclass
class Request:
    """
    Base class for all HTTP requests.

    Args:
        api_key (str): The authentication API key for API calls.
        url (str): URL of the API to call.
        headers (dict of str: str): The request headers.
        params (dict): The parameters to use in the API call.
        validator: The validator to use to validate params.
    """

    api_key: SecretStr
    url: HttpUrl
    headers: Dict[str, str]
    params: dict
    validator: Type[BaseModel]

    def __post_init__(self):
        """
        Validates self.params using the validator received in self.validator.
        """
        logger.debug("Request object received params: %s", self.params)
        self.params = self.validator(**self.params).dict(exclude_none=True)
        logger.debug("Request object params after validation: %s", self.params)

    def get(self):
        """
        Executes a GET request from the specified API.

        User's PDL_API_KEY is sent in request parameters.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        self.params["api_key"] = self.api_key
        logger.info(
            "Calling %s with params: %s",
            self.url,
            json.dumps(self.params, indent=2, default=utils.json_defaults),
        )
        self.params["api_key"] = self.params["api_key"].get_secret_value()
        return requests.get(self.url, params=self.params, headers=self.headers)

    def post(self):
        """
        Executes a POST request to the specified API.

        User's PDL_API_KEY is sent as a 'X-api-key' header.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        self.headers["X-api-key"] = self.api_key.get_secret_value()
        logger.info(
            "Calling %s with params: %s",
            self.url,
            json.dumps(self.params, indent=2, default=utils.json_defaults),
        )
        return requests.post(self.url, json=self.params, headers=self.headers)



"""
A settings singleton to share settings across different modules of the package.

Settings also loads environment variables eventually declared in an .env
file.
"""

import os

from dotenv import find_dotenv, load_dotenv
from pydantic import HttpUrl, SecretStr
from pydantic.dataclasses import dataclass


@dataclass
class Settings:
    """
    Singleton holding app's settings.

    Settings are eventually overridden if a .env file is provided, or
    environment variables are defined.
    """

    api_key: SecretStr = None
    base_path: HttpUrl = "https://api.peopledatalabs.com/"
    log_level: str = None
    log_format: str = "{asctime} [{levelname}] - {name}.{funcName}: {message}"
    version: str = "v5"
    version_re: str = r"^v[0-9]$"

    def __post_init__(self):
        load_dotenv(dotenv_path=find_dotenv(usecwd=True))
        for key in self.__dict__:
            if key == "api_key":
                self.api_key = os.getenv("PDL_API_KEY", self.api_key)
            else:
                self.__dict__[key] = os.getenv(key.upper(), self.__dict__[key])


settings = Settings()



"""
Models for input parameters of the Location APIs.
"""


from . import BaseRequestModel


class CleanerModel(BaseRequestModel):
    """
    Validation model for Location 'cleaner' API.
    """

    location: str



"""
Defines all API endpoints for the 'Person' section.
"""


from pydantic import (
    StrictStr,
    validate_arguments,
)
from pydantic.dataclasses import dataclass

from . import Endpoint
from .. import models
from ..models import person as person_models
from ..logger import get_logger


logger = get_logger("endpoints.person")


@dataclass
class Person(Endpoint):
    """
    Class for all APIs of "person" type.
    """

    section: str = "person"

    def bulk(self, **kwargs):
        """
        Calls PeopleDataLabs' person/bulk enrichment API.
        https://docs.peopledatalabs.com/docs/bulk-enrichment-api.

        Args:
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        return self._bulk(person_models.BulkModel, **kwargs)

    def enrichment(self, **kwargs):
        """
        Calls PeopleDataLabs' person/enrich API.
        https://docs.peopledatalabs.com/docs/enrichment-api.

        Args:
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        return self._enrichment(person_models.EnrichmentModel, **kwargs)

    def identify(self, **kwargs):
        """
        Calls PeopleDataLabs' person/identify API.
        https://docs.peopledatalabs.com/docs/identify-api.

        Args:
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        return self._identify(person_models.IdentifyModel, **kwargs)

    @validate_arguments
    def retrieve(self, person_id: StrictStr, **kwargs):
        """
        Calls PeopleDataLabs' person/retrieve API.
        https://docs.peopledatalabs.com/docs/person-retrieve-api.

        Args:
            person_id (str): The person's ID from the
                People Data Labs dataset;
            **kwargs: Additional parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        return self._retrieve(models.BaseRequestModel, person_id, **kwargs)

    def search(self, **kwargs):
        """
        Calls PeopleDataLabs' person/search API.
        https://docs.peopledatalabs.com/docs/search-api.

        Args:
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        return self._search(person_models.SearchModel, **kwargs)



"""
Defines all API endpoints for the 'Company' section.
"""


from pydantic.dataclasses import dataclass

from . import Endpoint
from ..models import company as company_models
from ..logger import get_logger


logger = get_logger("company")


@dataclass
class Company(Endpoint):
    """
    Class for all APIs of "company" type.
    """

    section: str = "company"

    def enrichment(self, **kwargs):
        """
        Calls PeopleDataLabs' company/enrich API.
        https://docs.peopledatalabs.com/docs/company-enrichment-api.

        Args:
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        return self._enrichment(company_models.EnrichmentModel, **kwargs)

    def search(self, **kwargs):
        """
        Calls PeopleDataLabs' company/search API.
        https://docs.peopledatalabs.com/docs/company-search-api.

        Args:
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        return self._search(company_models.SearchModel, **kwargs)

    def cleaner(self, **kwargs):
        """
        Calls PeopleDataLabs' company/clean API.

        Args:
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        return self._cleaner(company_models.CleanerModel, **kwargs)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['peopledatalabs', 'peopledatalabs.endpoints', 'peopledatalabs.models']

package_data = \
{'': ['*']}

install_requires = \
['email-validator>=1.1,<2.0',
 'pydantic>=1.8,<2.0',
 'python-dotenv>=0.13,<0.14',
 'requests>=2,<3']

setup_kwargs = {
    'name': 'peopledatalabs',
    'version': '1.0.4',
    'description': 'Official Python client for the People Data Labs API',
    'long_description': '<p align="center">\n<img src="https://i.imgur.com/S7DkZtr.png" width="250" alt="People Data Labs Logo">\n</p>\n<h1 align="center">People Data Labs Python Client</h1>\n<p align="center">Official Python client for the People Data Labs API.</p>\n\n<p align="center">\n  <a href="https://github.com/peopledatalabs/peopledatalabs-python">\n    <img src="https://img.shields.io/badge/repo%20status-Active-limegreen" alt="Repo Status">\n  </a>&nbsp;\n  <a href="https://pypi.org/project/peopledatalabs/">\n    <img src="https://img.shields.io/pypi/v/peopledatalabs.svg?logo=pypi&logoColor=fff&label=PyPI+package&color=limegreen" alt="People Data Labs on PyPI" />\n  </a>&nbsp;\n  <a href="https://pypi.org/project/peopledatalabs/">\n    <img src="https://img.shields.io/pypi/pyversions/peopledatalabs.svg" alt="People Data Labs on PyPI" />\n  </a>&nbsp;\n  <a href="https://github.com/peopledatalabs/peopledatalabs-python/actions/workflows/python-poetry.yml">\n    <img src="https://github.com/peopledatalabs/peopledatalabs-python/actions/workflows/python-poetry.yml/badge.svg" alt="Tests Status" />\n  </a>\n</p>\n\n## Table of Contents\n\n- [🔧 Installation](#installation)\n- [🚀 Usage](#usage)\n- [🌐 Endpoints](#endpoints)\n- [📘 Documentation](#documentation)\n\n## 🔧 Installation <a name="installation"></a>\n\n1. Install from PyPi using [pip](https://pip.pypa.io/en/latest/), a package manager for Python.\n\n    ```bash\n    pip install peopledatalabs\n    ```\n\n2. Sign up for a [free PDL API key](https://www.peopledatalabs.com/signup).\n3. Set your API key in the `PDL_API_KEY` environment variable.\n\n## 🚀 Usage <a name="usage"></a>\n\nFirst, create the PDLPY client:\n\n```python\nfrom peopledatalabs import PDLPY\n\n\n# specifying an API key\nclient = PDLPY(\n    api_key="YOUR API KEY",\n)\n\n# or leave blank if you have PDL_API_KEY set in your environment or .env file\nclient = PDLPY()\n```\n\n**Note:** You can provide your API key directly in code, or alternatively set a `PDL_API_KEY` variable in your environment or `.env` file.\n\nThen, send requests to any PDL API Endpoint.\n\n### Getting Person Data\n\n#### By Enrichment\n\n```python\nresult = client.person.enrichment(\n    phone="4155688415",\n    pretty=True,\n)\nif result.ok:\n    print(result.text)\nelse:\n    print(\n        f"Status: {result.status_code};"\n        f"\\nReason: {result.reason};"\n        f"\\nMessage: {result.json()[\'error\'][\'message\']};"\n    )\n```\n\n#### By Bulk Enrichment\n\n```python\nresult = client.person.bulk(\n    required="emails AND profiles",\n    requests=[\n        {\n            "metadata": {\n                "user_id": "123"\n            },\n            "params": {\n                "profile": ["linkedin.com/in/seanthorne"],\n                "location": ["SF Bay Area"],\n                "name": ["Sean F. Thorne"],\n            }\n        },\n        {\n            "metadata": {\n                "user_id": "345"\n            },\n            "params": {\n                "profile": ["https://www.linkedin.com/in/haydenconrad/"],\n                "first_name": "Hayden",\n                "last_name": "Conrad",\n            }\n        }\n    ]\n)\nif result.ok:\n    print(result.text)\nelse:\n    print(\n        f"Status: {result.status_code}"\n        f"\\nReason: {result.reason}"\n        f"\\nMessage: {result.json()[\'error\'][\'message\']}"\n    )\n```\n\n#### By Search (Elasticsearch)\n\n```python\nes_query = {\n    "query": {\n        "bool": {\n            "must": [\n                {"term": {"location_country": "mexico"}},\n                {"term": {"job_title_role": "health"}},\n            ]\n        }\n    }\n}\ndata = {\n    "query": es_query,\n    "size": 10,\n    "pretty": True,\n    "dataset": "phone, mobile_phone",\n}\nresult = client.person.search(**data)\nif result.ok:\n    print(result.text)\nelse:\n    print(\n        f"Status: {result.status_code}"\n        f"\\nReason: {result.reason}"\n        f"\\nMessage: {result.json()[\'error\'][\'message\']}"\n    )\n```\n\n#### By Search (SQL)\n\n```python\nsql_query = (\n    "SELECT * FROM person"\n    " WHERE location_country=\'mexico\'"\n    " AND job_title_role=\'health\'"\n    " AND phone_numbers IS NOT NULL;"\n)\ndata = {\n    "sql": sql_query,\n    "size": 10,\n    "pretty": True,\n    "dataset": "phone, mobile_phone",\n}\nresult = client.person.search(**data)\nif result.ok:\n    print(result.text)\nelse:\n    print(\n        f"Status: {result.status_code}"\n        f"\\nReason: {result.reason}"\n        f"\\nMessage: {result.json()[\'error\'][\'message\']}"\n    )\n```\n\n#### By `PDL_ID` (Retrieve API)\n\n```python\nresult = client.person.retrieve(\n    person_id="qEnOZ5Oh0poWnQ1luFBfVw_0000",\n)\nif result.ok:\n    print(result.text)\nelse:\n    print(\n        f"Status: {result.status_code}"\n        f"\\nReason: {result.reason}"\n        f"\\nMessage: {result.json()[\'error\'][\'message\']}"\n    )\n```\n\n#### By Fuzzy Enrichment (Identify API)\n\n```python\nresult = client.person.enrichment(\n    name="sean thorne",\n    pretty=True,\n)\nif result.ok:\n    print(result.text)\nelse:\n    print(\n        f"Status: {result.status_code}"\n        f"\\nReason: {result.reason}"\n        f"\\nMessage: {result.json()[\'error\'][\'message\']}"\n    )\n```\n\n### Getting Company Data\n\n#### By Enrichment\n\n```python\nresult = client.company.enrichment(\n    website="peopledatalabs.com",\n    pretty=True,\n)\nif result.ok:\n    print(result.text)\nelse:\n    print(\n        f"Status: {result.status_code}"\n        f"\\nReason: {result.reason}"\n        f"\\nMessage: {result.json()[\'error\'][\'message\']}"\n    )\n```\n\n#### By Search (Elasticsearch)\n\n```python\nes_query = {\n    "query": {\n        "bool": {\n            "must": [\n                {"term": {"tags": "big data"}},\n                {"term": {"industry": "financial services"}},\n                {"term": {"location.country": "united states"}},\n            ]\n        }\n    }\n}\ndata = {\n    "query": es_query,\n    "size": 10,\n    "pretty": True,\n}\nresult = client.company.search(**data)\nif result.ok:\n    print(result.text)\nelse:\n    print(\n        f"Status: {result.status_code}"\n        f"\\nReason: {result.reason}"\n        f"\\nMessage: {result.json()[\'error\'][\'message\']}"\n    )\n```\n\n#### By Search (SQL)\n\n```python\nsql_query = (\n    "SELECT * FROM company"\n    " WHERE tags=\'big data\'"\n    " AND industry=\'financial services\'"\n    " AND location.country=\'united states\';"\n)\ndata = {\n    "sql": sql_query,\n    "size": 10,\n    "pretty": True,\n}\nresult = client.company.search(**data)\nif result.ok:\n    print(result.text)\nelse:\n    print(\n        f"Status: {result.status_code}"\n        f"\\nReason: {result.reason}"\n        f"\\nMessage: {result.json()[\'error\'][\'message\']}"\n    )\n```\n\n### Using supporting APIs\n\n#### Get Autocomplete Suggestions\n\n```python\nresult = client.autocomplete(\n    field="title",\n    text="full",\n    size=10,\n)\nif result.ok:\n    print(result.text)\nelse:\n    print(\n        f"Status: {result.status_code}"\n        f"\\nReason: {result.reason}"\n        f"\\nMessage: {result.json()[\'error\'][\'message\']}"\n    )\n```\n\n#### Clean Raw Company Strings\n\n```python\nresult = client.company.cleaner(\n    name="peOple DaTa LabS",\n)\nif result.ok:\n    print(result.text)\nelse:\n    print(\n        f"Status: {result.status_code}"\n        f"\\nReason: {result.reason}"\n        f"\\nMessage: {result.json()[\'error\'][\'message\']}"\n    )\n```\n\n#### Clean Raw Location Strings\n\n```python\nresult = client.location.cleaner(\n    location="455 Market Street, San Francisco, California 94105, US",\n)\nif result.ok:\n    print(result.text)\nelse:\n    print(\n        f"Status: {result.status_code}"\n        f"\\nReason: {result.reason}"\n        f"\\nMessage: {result.json()[\'error\'][\'message\']}"\n    )\n```\n\n#### Clean Raw School Strings\n\n```python\nresult = client.school.cleaner(\n    name="university of oregon",\n)\nif result.ok:\n    print(result.text)\nelse:\n    print(\n        f"Status: {result.status_code}"\n        f"\\nReason: {result.reason}"\n        f"\\nMessage: {result.json()[\'error\'][\'message\']}"\n    )\n```\n\n## 🌐 Endpoints <a name="endpoints"></a>\n\n**Person Endpoints**\n\n| API Endpoint                                                                           | PDLPY Function                      |\n| -------------------------------------------------------------------------------------- | ----------------------------------- |\n| [Person Enrichment API](https://docs.peopledatalabs.com/docs/enrichment-api)           | `PDLPY.person.enrichment(**params)` |\n| [Person Bulk Enrichment API](https://docs.peopledatalabs.com/docs/bulk-enrichment-api) | `PDLPY.person.bulk(**params)`       |\n| [Person Search API](https://docs.peopledatalabs.com/docs/search-api)                   | `PDLPY.person.search(**params)`     |\n| [Person Retrieve API](https://docs.peopledatalabs.com/docs/person-retrieve-api)        | `PDLPY.person.retrieve(**params)`   |\n| [Person Identify API](https://docs.peopledatalabs.com/docs/identify-api)               | `PDLPY.person.identify(**params)`   |\n\n**Company Endpoints**\n\n| API Endpoint                                                                          | PDLPY Function                       |\n| ------------------------------------------------------------------------------------- | ------------------------------------ |\n| [Company Enrichment API](https://docs.peopledatalabs.com/docs/company-enrichment-api) | `PDLPY.company.enrichment(**params)` |\n| [Company Search API](https://docs.peopledatalabs.com/docs/company-search-api)         | `PDLPY.company.search(**params)`     |\n\n**Supporting Endpoints**\n\n| API Endpoint                                                                            | PDLJS Function                     |\n| --------------------------------------------------------------------------------------- | ---------------------------------- |\n| [Autocomplete API](https://docs.peopledatalabs.com/docs/autocomplete-api)               | `PDLPY.autocomplete(**params)`     |\n| [Company Cleaner API](https://docs.peopledatalabs.com/docs/cleaner-apis#companyclean)   | `PDLPY.company.cleaner(**params)`  |\n| [Location Cleaner API](https://docs.peopledatalabs.com/docs/cleaner-apis#locationclean) | `PDLPY.location.cleaner(**params)` |\n| [School Cleaner API](https://docs.peopledatalabs.com/docs/cleaner-apis#schoolclean)     | `PDLPY.school.cleaner(**params)`   |\n\n\n## 📘 Documentation <a name="documentation"></a>\n\nAll of our API endpoints are documented at: https://docs.peopledatalabs.com/\n\nThese docs describe the supported input parameters, output responses and also provide additional technical context.\n\nAs illustrated in the [Endpoints](#endpoints) section above, each of our API endpoints is mapped to a specific method in the PDLPY class.  For each of these class methods, **all function inputs are mapped as input parameters to the respective API endpoint**, meaning that you can use the API documentation linked above to determine the input parameters for each endpoint.\n\nAs an example:\n\nThe following is **valid** because `name` is a [supported input parameter to the Person Identify API](https://docs.peopledatalabs.com/docs/identify-api-reference#input-parameters):\n\n```python\nPDLPY().person.identify({"name": "sean thorne"})\n```\n\nConversely, this would be **invalid** because `fake_parameter` is not an input parameter to the Person Identify API:\n\n```python\nPDLPY().person.identify({"fake_parameter": "anything"})\n```\n',
    'author': 'People Data Labs',
    'author_email': 'hello@peopledatalabs.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://www.peopledatalabs.com',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.2,<4.0.0',
}


setup(**setup_kwargs)
